//#############################################################################################################################
//
//	AY Interface by Mike Dailly - No copyright or license, do whatever you want with it.
//
//#############################################################################################################################
#include "ayemu.h"
#include <memory.h>

#define	MAX_AY	3


unsigned char*	pAYBuffer[MAX_AY];
int				bufindex[MAX_AY];
ayemu_ay_t		AY[MAX_AY];


//#############################################################################################################################
//
//	Function:		InitAY
//	Description:	Initialise an AY chip (upto MAX_AY)
//
//	In:				AY chip to init (0 to MAX_AY-1)
//					Sample buffer to build into
//
//#############################################################################################################################
extern "C" __declspec(dllexport) void	InitAY(int _ay, unsigned char* _pAYBuffer)
{
	// remember buffer
	pAYBuffer[_ay] = _pAYBuffer;

	// init AY structure and reset virtual chip
	memset(&(AY[_ay]), 0, sizeof(ayemu_ay_t));
	ayemu_init( &(AY[_ay]) );
	ayemu_reset(&(AY[_ay]));

	// will create a 1248 buffer per frame (624 samples at 16bit)
	int ret = ayemu_set_sound_format(&(AY[_ay]), 312 * 2 * 50, 1, 16);		
	
	// reset index
	bufindex[_ay] = 0;
}

//#############################################################################################################################
//
//	Function:		WriteAY
//	Description:	Writes to an AY register
//
//	In:				AY chip to write to(0 to MAX_AY-1)
//					register to write in that chip
//					value to write to register
//
//#############################################################################################################################
extern "C" __declspec(dllexport)  void	WriteAY(int _ay, unsigned int _reg, unsigned int _value8) {
	
	ayemu_set_reg(&(AY[_ay]), _reg, _value8);
}

//#############################################################################################################################
//
//	Function:		ProcessAY
//	Description:	"tick" AY for 1 sample. Will be called througout the frame - twice per virtual scanline
//
//	In:				AY chip to process
//					index into sample (0 to 623)
//
//#############################################################################################################################
extern "C"  __declspec(dllexport)  void	ProcessAY(int _ay,int index)
{
	void *pData;
	pData = ayemu_gen_sound(&(AY[_ay]), &(pAYBuffer[_ay][index*2]), 1);
}


//#############################################################################################################################
//
//	Function:		AYFrameReset
//	Description:	Reset the sample index back to 0. Should be called at the start 
//					of each virtual frame (for each chip enabled)
//
//	In:				AY chip to process
//
//#############################################################################################################################
extern "C"  __declspec(dllexport)  void	AYFrameReset(int _ay)
{
	bufindex[_ay] = 0;
}
